@@ -61,11 +61,13 @@ def table_row( title:str, button_htmls:List[str] = []) -> str:
   Uses the provided title and list of button html strings to
   return the row of an AL document-styled table in HTML format.
   """
-  html = '\n\t<tr>'
-  # html += '\n\t\t<td><i class="fas fa-file"></i>&nbsp;&nbsp;</td>'
-  # TODO: Need to replace with proper CSS
-  html += '\n\t\t<td class="al_doc_title"><strong>' + title + '</strong></td>'
-  html += '\n\t\t<td class="al_buttons">'
+  html = (
+    f'\n\t<tr>'
+    # '\n\t\t<td><i class="fas fa-file"></i>&nbsp;&nbsp;</td>'
+    # TODO: Need to replace with proper CSS
+    f'\n\t\t<td class="al_doc_title"><strong>{title}</strong></td>'
+    f'\n\t\t<td class="al_buttons">'
+  )
   for button in button_htmls:
     html += button
   html += '</td>'
@@ -781,11 +783,11 @@ class ALDocumentBundle(DAList):
     files = self.enabled_documents(refresh=refresh)
     if len(files) == 1:
       # This case is simplest--we do not need to process the document at this level
-      log_if_debug('Storing bundle for just one document ' + self.title + ' at ' + self.instanceName + '.cache.' + safe_key)
+      log_if_debug(f'Storing bundle for just one document {self.title} at {self.instanceName}.cache.{safe_key}')
       pdf = files[0].as_pdf(key=key, refresh=refresh)
       pdf.title = self.title
     else:
-      log_if_debug('Storing bundle ' + self.title + ' at ' + self.instanceName + '.cache.' + safe_key)
+      log_if_debug(f'Storing bundle {self.title} at {self.instanceName}.cache.{safe_key}')
       pdf = pdf_concatenate([document.as_pdf(key=key, refresh=refresh) for document in files], filename=self.filename + ending)
     pdf.title = self.title
     setattr(self.cache, safe_key, pdf)
@@ -843,7 +845,8 @@ class ALDocumentBundle(DAList):
           retval.append(document)
       return retval
     else:
-      return [document for document in self.elements if document.enabled]
+      return [document for document in self.elements 
+          if (hasattr(document, 'always_enabled') and document.always_enabled) or document.enabled]
 
   def as_flat_list(self, key:str='final', refresh:bool=True) -> List[DAFile]:
     """
@@ -862,15 +865,15 @@ class ALDocumentBundle(DAList):
         flat_list.extend(document.as_list(key=key, refresh=refresh))
     return flat_list
 
-  def get_titles(self, key:str='final') -> List[str]:
+  def get_titles(self, key:str='final', refresh:bool=True) -> List[str]:
     """
     Gets all of titles of the documents in a list
     """
     flat_list = []
-    for document in self:
+    for document in self.enabled_documents(refresh=refresh):
       if isinstance(document, ALDocumentBundle):
-        flat_list.extend(document.get_titles(key=key))
-      elif document.enabled:
+        flat_list.extend(document.get_titles(key=key, refresh=refresh))
+      else:
         flat_list.append(document.title)
     return flat_list
 
@@ -976,9 +979,11 @@ class ALDocumentBundle(DAList):
     else:
       buttons = [doc_download_button]
 
-    html ='<table class="al_table merged_docs" id="' + html_safe_str(self.instanceName) + '">'
-    html += table_row(self.title, buttons)
-    html += '\n</table>'
+    html = ( 
+      f'<table class="al_table merged_docs" id="{html_safe_str(self.instanceName)}">'
+      f'{table_row(self.title, buttons)}'
+      f'\n</table>'
+    )
 
     return html
 
@@ -990,37 +995,38 @@ class ALDocumentBundle(DAList):
     Optionally, display a checkbox that allows someone to decide whether or not to
     include an editable (Word) copy of the file, iff it is available.
     """
-    name = re.sub(r'[^A-Za-z0-9]+','_', self.instanceName)  # safe name for classes and ids
+    name = html_safe_str(self.instanceName)  
     al_wants_editable_input_id = '_ignore_al_wants_editable_' + name
     al_email_input_id = '_ignore_al_doc_email_' + name
-    al_send_button_id = "al_send_email_button_" + name
+    al_send_button_id = 'al_send_email_button_' + name
 
-    javascript_string = "javascript:aldocument_send_action('" + \
-      self.attr_name('send_email_action_event') + \
-      "','" + al_wants_editable_input_id + "','" + \
-      al_email_input_id + "')"
+    javascript_string = (
+      f"javascript:aldocument_send_action("
+      f"'{self.attr_name('send_email_action_event')}',"
+      f"'{al_wants_editable_input_id}','{al_email_input_id}')"
+    )
 
-    return_str = '''
-  <div class="al_send_bundle '''+name+'''" id="al_send_bundle_'''+name+'''" name="al_send_bundle_'''+name+'''">
+    return_str = f'''
+  <div class="al_send_bundle {name}" id="al_send_bundle_{name}" name="al_send_bundle_{name}">
     <h5 id="al_doc_email_header">Get a copy of the documents in email</h5> 
     '''
     if show_editable_checkbox:
-      return_str += '''
+      return_str += f'''
     <div class="form-check-container">
       <div class="form-check">
-        <input class="form-check-input" type="checkbox" class="al_wants_editable" id="'''+al_wants_editable_input_id+'''">
-        <label class="al_wants_editable form-check-label" for="'''+al_wants_editable_input_id+'''">'''\
-          + word("Include an editable copy") + '''
+        <input class="form-check-input" type="checkbox" class="al_wants_editable" id="{al_wants_editable_input_id}">
+        <label class="al_wants_editable form-check-label" for="{al_wants_editable_input_id}">{word("Include an editable copy")}
         </label>
       </div>
     </div>
   '''
-    return_str += '''
+    return_str += f'''
   <div class="al_email_container">
-    <span class="al_email_address '''+name+''' form-group row da-field-container da-field-container-datatype-email">
-      <label for="'''+al_email_input_id+'''" class="al_doc_email col-form-label da-form-label datext-right">Email</label>
-      <input value="''' + (user_info().email if user_logged_in() else '') + '" alt="Input box" class="form-control" type="email" size="35" name="'+al_email_input_id+'" id="'+al_email_input_id+'''">
-    </span>''' + action_button_html(javascript_string, label="Send", icon="envelope", color="primary", size="md", classname="al_send_email_button", id_tag=al_send_button_id) + "\n" + '''
+    <span class="al_email_address {name} form-group row da-field-container da-field-container-datatype-email">
+      <label for="{al_email_input_id}" class="al_doc_email col-form-label da-form-label datext-right">Email</label>
+      <input value="{user_info().email if user_logged_in() else ''}" alt="Input box" class="form-control" type="email" size="35" name="{al_email_input_id}" id="{al_email_input_id}">
+    </span>{action_button_html(javascript_string, label="Send", icon="envelope", color="primary", size="md", classname="al_send_email_button", id_tag=al_send_button_id)}
+
   </div>
   '''
     return_str += '</div>'  # al_send_bundle
@@ -1238,9 +1244,10 @@ class ALExhibitList(DAList):
     """
     Private method automatically triggered when the list is fully gathered.
     """
-    self._update_page_numbers()
-    if self.auto_label:
-      self._update_labels()
+    if len(self):
+      self._update_page_numbers()
+      if self.auto_label:
+        self._update_labels()
     
     # TODO: implement below. Was buggy when OCRing a lot of files and OCRing in place. 
     # Switched to creating a new DAFile, but that introduced different bugs. -- third refresh breaks it. file never gets a number so not "ok"
@@ -1271,7 +1278,9 @@ class ALExhibitDocument(ALDocument):
     - exhibit_attachment: ALExhibitDocument.using(title="Exhibits", filename="exhibits" )
   ---
   code: |
-    exhibit_attachment.enabled = exhibit_attachment.exhibits.there_are_any
+    # This block is not needed, but you can provide and customize for your needs.
+    # This mirrors the fallback block in ql_baseline.yml
+    exhibit_attachment.enabled = exhibit_attachment.exhibits.has_exhibits 
   ---
   objects:
     - al_user_bundle: ALDocumentBundle.using(elements=[my_instructions, my_main_attachment, exhibit_attachment], filename="user_bundle.pdf", title="All forms to download for your records")
@@ -1308,7 +1317,8 @@ class ALExhibitDocument(ALDocument):
     """
     filename = os.path.splitext(self.filename)[0] + ".pdf"
     
-    return pdf_concatenate(self.table_of_contents, self.exhibits.as_pdf(add_cover_pages=self.include_exhibit_cover_pages), filename=filename)
+    if len(self.exhibits):
+      return pdf_concatenate(self.table_of_contents, self.exhibits.as_pdf(add_cover_pages=self.include_exhibit_cover_pages), filename=filename)    
     # pdf_concatenate([a.as_pdf() for a in self.exhibits], filename=self.filename)
 
   def as_docx(self, key:str="bool", refresh:bool=True) -> DAFile:
@@ -1326,4 +1336,4 @@ def unpack_dafilelist(the_file:DAFileList)->DAFile:
     temp_name = the_file.instanceName
     the_file = next(iter(the_file))
     the_file.instanceName = temp_name # reset instance name to the whole object instead of index in list we got rid of
-  return the_file
\ No newline at end of file
+  return the_file
